* gllamm review by L. Grilli & C. Rampichini at http://www.mlwin.com/softrev/gllamm.shtml
* March 2006
* gllamm rel. 2.3.11 of 15/10/2005

* Section 2.1 TWO-LEVEL NORMAL MODELS

*define the working directory
cd D:\Documenti\Lavori_in_corso\gllamm_review

log using Exam.log, text

*read data (assuming they are in the working directory)
infile schid studid exam const standlrt gender schgend schlrt bandschlrt bandstudlrt using Exam.txt

*assign labels 
label variable schid  "School ID"
label variable studid  "Student ID" 
label variable exam "Normalised exam score as outcome variable"
label variable standlrt  "Standardised LR test score as intake variable"
label variable gender  "Student gender (0 = boys, 1 = girls)"
label variable schgend   "School gender (1 = mixed school, 2 = boys school, 3 = girls school)"
label variable schlrt "School average of intake score"

*Define value label
label define gender 0 "0 boys"  1 "1 girls", modify
label define schgend 1 "1 mixed school"  2 "2 boys school"  3 "3 girls school", modify

*Assign value label to variable
label values gender gender 
label values schgend schgend

*reference category for schgend
char schgend[omit] 3


******************************************
*** model A: variance component model  ***
******************************************

*REML estimate
xi: xtmixed exam standlrt gender i.schgend || schid: , variance

*see names of estimated parameters
matrix list e(b)
*compute variance components from model parameters
nlcom (var_u0: exp(2*[lns1_1_1]_cons) ) (var_e: exp(2*[lnsig_e]_cons))

*ML estimate with xtmixed
xi: xtmixed exam standlrt gender i.schgend || schid:, ml variance

*ML estimate with gllamm (8 quadrature point adaptive)
xi: gllamm exam standlrt gender i.schgend, i(schid) adapt nip(8) trace
*save model estimates
estimates store mod_a

matrix b=e(b)
*estimate again with more quadrature points, keep initial values from previous model
xi: gllamm exam standlrt gender i.schgend, i(schid) adapt nip(12) from(b)

*empirical Bayes predictions of random effects
estimates restore mod_a
gllapred eb, u adapt

*summarize the values of ebm1 for the schools:
*(1) create a dummy f equal to one for only one observation in each schooland (2) summarize
sort schid
qui by schid: gen f=_n==1
summ ebm1  if f==1


***********************************************************************************************
*** model A with NPML: variance component model with discrete marginal distribution of u_0j ***
***********************************************************************************************

xi: gllamm exam standlrt gender i.schgend, i(schid) ip(f) nip(2) 

*save parameter estimates
matrix b=e(b)
*save loglike
local ll=e(ll)
*save number of estimated parameters
local k=e(k)
*estimate again with a higer number of mass points
*WARNING: THE PREVIOUS LINES DEFINING THE LOCAL VARIABLES ll AND k MUST BE  
*SUBMITTED JOINTLY WITH THE FOLLOWING LINE CALLING GLLAMM
xi: gllamm exam standlrt gender i.schgend, i(schid) ip(f) nip(3) from(b) gateaux(-2, 2, 30) lf0(`k' `ll')
estimates store mod_astar3

*save results of the model
matrix b=e(b)
local ll=e(ll)
local k=e(k)
*estimate again with a higer number of mass points
xi: gllamm exam standlrt gender i.schgend, i(schid) ip(f) nip(4) from(b) gateaux(-2, 2, 30) lf0(`k' `ll')
estimates store mod_astar4

*save results of the model
matrix b=e(b)
local ll=e(ll)
local k=e(k)
*estimate again with a higer number of mass points
xi: gllamm exam standlrt gender i.schgend, i(schid) ip(f) nip(5) from(b) gateaux(-2, 2, 30) lf0(`k' `ll')
estimates store mod_astar5

*save results of the model
matrix b=e(b)
local ll=e(ll)
local k=e(k)
*estimate again with a higer number of mass points
xi: gllamm exam standlrt gender i.schgend, i(schid) ip(f) nip(6) from(b) gateaux(-2, 2, 30) lf0(`k' `ll')

estimates stats


**********************************************************
*** model B: variance component model with interaction ***
**********************************************************

*REML estimte
xi: xtmixed exam  i.gender*standlrt  i.schgend || schid: , variance

*******************************************
*** model C: random slope for standlrt  ***
***       correlated random effects     ***
*******************************************

*REML estimate
xi: xtmixed exam i.gender*standlrt i.schgend || schid:standlrt, cov(unstruct) variance

*ML estimate via gllamm (useful as initial values for model D)
eq sch_c: cons
eq sch_lrt: standlrt
xi: gllamm exam i.gender*standlrt i.schgend , i(schid)  nrf(2) eqs(sch_c sch_lrt) adapt nip(5) trace dots
matrix b=e(b)


*************************************************
*** model D: level 1 heterogeneity for gender ***
*************************************************

eq sch_c: cons
eq sch_lrt: standlrt

gen boy=gender==0
gen girl=gender==1

eq vargen: boy girl 

*adaptive quadrature
xi: gllamm exam i.gender*standlrt i.schgend , i(schid) nrf(2) eqs(sch_c sch_lrt) /*
	*/ s(vargen) adapt nip(8) trace dots

*spherical adaptive quadrature
xi: gllamm exam i.gender*standlrt i.schgend , i(schid) nrf(2) eqs(sch_c sch_lrt) /*
	*/ s(vargen) ip(m) adapt nip(7) trace dots
estimates store mod_d

*calculates first level variances and their standard errors
matrix list e(b)
nlcom (s_boy: exp(2*[lns1]boy))  (s_girl: exp(2*[lns1]girl))


***************************************************
*** model E: level 1 heterogeneity for standlrt ***
***************************************************

eq varlrt: cons standlrt 
xi: gllamm exam i.gender*standlrt i.schgend , i(schid) nrf(2) eqs(sch_c sch_lrt) /*
	*/ s(varlrt) ip(m) adapt nip(7) trace dots
estimates store mod_e

*calculates first level variances and their standard errors
matrix list e(b)
nlcom (s_e: exp(2*[lns1]const ))  (delta: 2*[lns1]standlrt)


**************************
*** linear constraints ***
**************************

constraint define 1 [lns1]_cons=0
constraint define 2 _Ischgend_1 =_Ischgend_2 

*model A
xi: gllamm exam standlrt gender i.schgend, i(schid) constraint(1 2) adapt nip(8) trace


