* gllamm review by L. Grilli & C. Rampichini at http://www.mlwin.com/softrev/gllamm.shtml
* March 2006
* gllamm rel. 2.3.11 of 15/10/2005

* Sections 3.2 MULTIVARIATE NORMAL RESPONSE MODEL & 3.3 MULTIVARIATE BINARY-NORMAL RESPONSE MODEL

*define the working directory
cd D:\Documenti\Lavori_in_corso\gllamm_review

log using Multivariate.log, text

*The data contain GCSE exam scores on a science subject
*Two components of the exam were considered as outcome variables: written paper and course work
*There were 1,905 students from 73 schools in England in the data. 

*read data (assuming they are in the working directory)
infile school student gender y1 y2 using Gcsemv.txt

*assign labels to describe the variables
label variable school  	"School ID"
label variable student  "Student ID" 
label variable gender  	"Student gender (0 = boys, 1 = girls)"
label variable y1   	"Total score of written papers"
label variable y2 	"Total score of course work"

*define value label
label define gender_lab 0 "0 boys"  1 "1 girls"
*assign value label "gender_lab" to variable "gender"
label values gender gender_lab 

*Recoding missing values (from -1 to .)
replace y1=. if y1==-1
replace y2=. if y2==-1

*create a unique code for students
egen id = group(school student)

* Reshaping data in long form
reshape long y, i(id) j(index)

generate index1=(index==1)
generate index2=(index==2)

generate gender1=gender*index1
generate gender2=gender*index2


*********************************************************************
*** Section 3.3 MULTIVARIATE NORMAL RESPONSE MODEL                ***
*** 3 level normal-normal model with constrained level 1 variance ***
*********************************************************************

eq index1: index1
eq index2: index2

constraint define 1 [lns1]_cons=1.5

*model fitted with 7-point adaptive sherical quadrature (Table 7, first column)

gllamm y index1 index2 gender1 gender2, nocons i(id school) nrf(2 2) /*
	*/ eqs(index1 index2 index1 index2) constraint(1) ip(m) nip(7) adapt trace 


******************************************************************
*** Section 3.3 MULTIVARIATE NORMAL RESPONSE MODEL             ***
*** 3 level normal-normal model with a factor at student level ***
******************************************************************

eq index1: index1
eq index2: index2
eq factoreq: index1 index2

gllamm y index1 index2 gender1 gender2, nocons i(id school) /* 
        */ nrf(1 2) eqs(factoreq index1 index2) ip(m) nip(7) adapt

*compute student level variances

nlcom var1: [id1_1]index1^2+exp(2*[lns1]_cons)
nlcom var2: [id1_1]index1^2*[id1_1l]index2^2+exp(2*[lns1]_cons)

*compute student level covariance and correlation

nlcom cov: [id1_1l]index2*([id1_1]index1)^2
nlcom rho: ([id1_1l]index2*[id1_1]index1^2)/sqrt(([id1_1]index1^2+exp(2*[lns1]_cons)) /*
                 */ *([id1_1]index1^2*[id1_1l]index2^2+exp(2*[lns1]_cons)))


******************************************************************************************
*** Section 3.4 MULTIVARIATE BINARY-NORMAL RESPONSE MODEL                              ***
*** two-level bivariate mixed model with index1 binomial and a factor at student level ***
******************************************************************************************

*recode y1 as binary
gen ymixed=y
replace ymixed=1 if (index1==1 & y>80)
replace ymixed=0 if (index1==1 & y <= 80)
replace ymixed=. if y==.

eq factoreq: index1 index2

constraint define 2 [id1_1]index1=1

gllamm ymixed index1 index2, nocons i(id) nrf(1) eqs(factoreq) constraint(2) /*
           */ lv(index) link(logit identity) fv(index) family(binomial gaussian) nip(10) adapt


*compute the variance of the continuous response (index2)
nlcom var2: [id1_1l]index2^2+exp(2*[lns1]_cons)
*compute the correlation among the responses
nlcom rho: [id1_1l]index2/sqrt((1+(_pi^2/3))*[id1_1l]index2^2+exp(2*[lns1]_cons))


